use crate::cli::prelude::*;
use crate::prelude::*;
use crate::state::State;

/// Run the web application server
#[derive(Args)]
pub struct Run {
    /// Whether or not to watch certain resource files for changes and reload accordingly
    #[arg(short, long, default_value = None)]
    pub watch: bool,

    /// The address to bind to - you almost certainly want to use :: or 0.0.0.0 instead of the default
    #[arg(short = 'H', long, default_value = "::1")]
    pub host: String,

    /// The port to bind to
    #[arg(short, long, default_value = "3000")]
    pub port: u16,
    // The database connection string to use
    // #[arg(long, default_value = "postgresql://lyrs?host=/var/run/postgresql")]
    // pub database_connection_string: String,

    // TODO: disallow in production?
    // Delete all data, recreate new database, run migrations, and seed database
    // #[arg(long, default_value = "false")]
    // pub database_reset: bool,
}

impl Run {
    pub async fn run(&self) -> AnyResult<()> {
        let app_state = State::try_new().await?;
        let (router, _watchers) = crate::router::router(app_state, self.watch).await?;
        Ok(
            crate::webserver::webserver(router, self.watch, Some(&self.host), Some(self.port))
                .await?,
        )
    }
}
